//-----------------------------------------------------
// name: "Modal Bar"
// author: "Romain Michon (rmichon@ccrma.stanford.edu)"
// copyright: "Romain Michon"
// version: "1.0"
//
// Code generated with Faust 0.9.59 (http://faust.grame.fr)
//-----------------------------------------------------
/* link with : "" */
#include "instrument.h"
#include <math.h>
#include "modalBar.h"
#include <cmath>
template <int N> inline float faustpower(float x) 		{ return powf(x,N); } 
template <int N> inline double faustpower(double x) 	{ return pow(x,N); }
template <int N> inline int faustpower(int x) 			{ return faustpower<N/2>(x) * faustpower<N-N/2>(x); } 
template <> 	 inline int faustpower<0>(int x) 		{ return 1; }
template <> 	 inline int faustpower<1>(int x) 		{ return x; }
/************************************************************************
 ************************************************************************
    FAUST Architecture File
	Copyright (C) 2006-2011 Albert Graef <Dr.Graef@t-online.de>
    ---------------------------------------------------------------------
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as 
	published by the Free Software Foundation; either version 2.1 of the 
	License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
 	License along with the GNU C Library; if not, write to the Free
  	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  	02111-1307 USA. 
 ************************************************************************
 ************************************************************************/

/* Pd architecture file, written by Albert Graef <Dr.Graef@t-online.de>.
   This was derived from minimal.cpp included in the Faust distribution.
   Please note that this is to be compiled as a shared library, which is
   then loaded dynamically by Pd as an external. */

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <string>

#include "faust/misc.h"
#include "faust/gui/UI.h"
#include "faust/gui/meta.h"
#include "faust/audio/dsp.h"

using namespace std;

/******************************************************************************
*******************************************************************************

							       VECTOR INTRINSICS

*******************************************************************************
*******************************************************************************/


/***************************************************************************
   Pd UI interface
 ***************************************************************************/

enum ui_elem_type_t {
  UI_BUTTON, UI_CHECK_BUTTON,
  UI_V_SLIDER, UI_H_SLIDER, UI_NUM_ENTRY,
  UI_V_BARGRAPH, UI_H_BARGRAPH,
  UI_END_GROUP, UI_V_GROUP, UI_H_GROUP, UI_T_GROUP
};

struct ui_elem_t {
  ui_elem_type_t type;
  char *label;
  float *zone;
  float init, min, max, step;
};

class PdUI : public UI
{
public:
  int nelems;
  ui_elem_t *elems;
		
  PdUI();
  PdUI(const char *s);
  virtual ~PdUI();

protected:
  string path;
  void add_elem(ui_elem_type_t type, const char *label = NULL);
  void add_elem(ui_elem_type_t type, const char *label, float *zone);
  void add_elem(ui_elem_type_t type, const char *label, float *zone,
		float init, float min, float max, float step);
  void add_elem(ui_elem_type_t type, const char *label, float *zone,
		float min, float max);

public:
  virtual void addButton(const char* label, float* zone);
  virtual void addCheckButton(const char* label, float* zone);
  virtual void addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step);
  virtual void addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step);
  virtual void addNumEntry(const char* label, float* zone, float init, float min, float max, float step);

  virtual void addHorizontalBargraph(const char* label, float* zone, float min, float max);
  virtual void addVerticalBargraph(const char* label, float* zone, float min, float max);
  
  virtual void openTabBox(const char* label);
  virtual void openHorizontalBox(const char* label);
  virtual void openVerticalBox(const char* label);
  virtual void closeBox();
	
  virtual void run();
};

static string mangle(const char *s)
{
  const char *s0 = s;
  string t = "";
  if (!s) return t;
  while (*s)
    if (isalnum(*s))
      t += *(s++);
    else {
      const char *s1 = s;
      while (*s && !isalnum(*s)) ++s;
      if (s1 != s0 && *s) t += "-";
    }
  return t;
}

static string normpath(string path)
{
  path = string("/")+path;
  int pos = path.find("//");
  while (pos >= 0) {
    path.erase(pos, 1);
    pos = path.find("//");
  }
  return path;
}

static string pathcat(string path, string label)
{
  if (path.empty())
    return normpath(label);
  else if (label.empty())
    return normpath(path);
  else
    return normpath(path+"/"+label);
}

PdUI::PdUI()
{
  nelems = 0;
  elems = NULL;
  path = "";
}

PdUI::PdUI(const char *s)
{
  nelems = 0;
  elems = NULL;
  path = s?s:"";
}

PdUI::~PdUI()
{
  if (elems) {
    for (int i = 0; i < nelems; i++)
      if (elems[i].label)
	free(elems[i].label);
    free(elems);
  }
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  string s = pathcat(path, mangle(label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = NULL;
  elems[nelems].init = 0.0;
  elems[nelems].min = 0.0;
  elems[nelems].max = 0.0;
  elems[nelems].step = 0.0;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  string s = pathcat(path, mangle(label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = 0.0;
  elems[nelems].min = 0.0;
  elems[nelems].max = 1.0;
  elems[nelems].step = 1.0;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone,
			  float init, float min, float max, float step)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  string s = pathcat(path, mangle(label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = init;
  elems[nelems].min = min;
  elems[nelems].max = max;
  elems[nelems].step = step;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone,
			  float min, float max)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  string s = pathcat(path, mangle(label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = 0.0;
  elems[nelems].min = min;
  elems[nelems].max = max;
  elems[nelems].step = 0.0;
  nelems++;
}

void PdUI::addButton(const char* label, float* zone)
{ add_elem(UI_BUTTON, label, zone); }
void PdUI::addCheckButton(const char* label, float* zone)
{ add_elem(UI_CHECK_BUTTON, label, zone); }
void PdUI::addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_V_SLIDER, label, zone, init, min, max, step); }
void PdUI::addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_H_SLIDER, label, zone, init, min, max, step); }
void PdUI::addNumEntry(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_NUM_ENTRY, label, zone, init, min, max, step); }

void PdUI::addHorizontalBargraph(const char* label, float* zone, float min, float max)
{ add_elem(UI_H_BARGRAPH, label, zone, min, max); }
void PdUI::addVerticalBargraph(const char* label, float* zone, float min, float max)
{ add_elem(UI_V_BARGRAPH, label, zone, min, max); }

void PdUI::openTabBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(label);
}
void PdUI::openHorizontalBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(label);
}
void PdUI::openVerticalBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(label);
}
void PdUI::closeBox()
{
  int pos = path.rfind("/");
  if (pos < 0) pos = 0;
  path.erase(pos);
}

void PdUI::run() {}

/******************************************************************************
*******************************************************************************

			    FAUST DSP

*******************************************************************************
*******************************************************************************/

//----------------------------------------------------------------------------
//  FAUST generated signal processor
//----------------------------------------------------------------------------
		
#ifndef FAUSTFLOAT
#define FAUSTFLOAT float
#endif  

typedef long double quad;

#ifndef FAUSTCLASS 
#define FAUSTCLASS mydsp
#endif

class mydsp : public dsp {
  private:
	class SIG0 {
	  private:
		int 	fSamplingFreq;
		int 	iRec13[2];
	  public:
		int getNumInputs() 	{ return 0; }
		int getNumOutputs() 	{ return 1; }
		void init(int samplingFreq) {
			fSamplingFreq = samplingFreq;
			for (int i=0; i<2; i++) iRec13[i] = 0;
		}
		void fill (int count, float output[]) {
			for (int i=0; i<count; i++) {
				iRec13[0] = (1 + iRec13[1]);
				output[i] = readMarmstk1(int(((iRec13[0] - 1) % 246)));
				// post processing
				iRec13[1] = iRec13[0];
			}
		}
	};


	FAUSTFLOAT 	fslider0;
	int 	iConst0;
	float 	fConst1;
	float 	fConst2;
	float 	fConst3;
	float 	fConst4;
	float 	fConst5;
	float 	fConst6;
	float 	fConst7;
	float 	fConst8;
	float 	fRec11[2];
	float 	fConst9;
	float 	fRec10[2];
	int 	IOTA;
	float 	fVec0[8192];
	float 	fConst10;
	int 	iConst11;
	static float 	ftbl0[246];
	FAUSTFLOAT 	fslider1;
	float 	fConst12;
	FAUSTFLOAT 	fslider2;
	float 	fRec14[2];
	float 	fConst13;
	float 	fConst14;
	float 	fConst15;
	float 	fConst16;
	float 	fConst17;
	float 	fConst18;
	FAUSTFLOAT 	fslider3;
	FAUSTFLOAT 	fbutton0;
	float 	fRec16[3];
	float 	fConst19;
	float 	fRec15[2];
	FAUSTFLOAT 	fbutton1;
	FAUSTFLOAT 	fslider4;
	float 	fRec17[2];
	float 	fConst20;
	float 	fRec18[2];
	int 	iRec19[2];
	float 	fRec20[2];
	float 	fRec12[3];
	float 	fConst21;
	float 	fRec22[2];
	float 	fRec23[2];
	float 	fRec21[3];
	float 	fConst22;
	float 	fRec25[2];
	float 	fRec26[2];
	float 	fRec24[3];
	float 	fConst23;
	float 	fRec28[2];
	float 	fRec29[2];
	float 	fRec27[3];
	float 	fRec30[3];
	float 	fRec31[3];
	float 	fRec32[3];
	float 	fRec33[3];
	float 	fConst24;
	float 	fVec1[4096];
	int 	iConst25;
	float 	fVec2[2048];
	int 	iConst26;
	float 	fRec8[2];
	float 	fConst27;
	float 	fConst28;
	float 	fConst29;
	float 	fRec37[2];
	float 	fConst30;
	float 	fRec36[2];
	float 	fVec3[8192];
	float 	fConst31;
	int 	iConst32;
	float 	fVec4[1024];
	int 	iConst33;
	float 	fRec34[2];
	float 	fConst34;
	float 	fConst35;
	float 	fConst36;
	float 	fRec41[2];
	float 	fConst37;
	float 	fRec40[2];
	float 	fVec5[8192];
	float 	fConst38;
	int 	iConst39;
	float 	fVec6[2048];
	int 	iConst40;
	float 	fRec38[2];
	float 	fConst41;
	float 	fConst42;
	float 	fConst43;
	float 	fRec45[2];
	float 	fConst44;
	float 	fRec44[2];
	float 	fVec7[8192];
	float 	fConst45;
	int 	iConst46;
	float 	fVec8[1024];
	int 	iConst47;
	float 	fRec42[2];
	float 	fConst48;
	float 	fConst49;
	float 	fConst50;
	float 	fRec49[2];
	float 	fConst51;
	float 	fRec48[2];
	float 	fVec9[16384];
	float 	fConst52;
	int 	iConst53;
	float 	fVec10[2048];
	int 	iConst54;
	float 	fRec46[2];
	float 	fConst55;
	float 	fConst56;
	float 	fConst57;
	float 	fRec53[2];
	float 	fConst58;
	float 	fRec52[2];
	float 	fVec11[8192];
	float 	fConst59;
	int 	iConst60;
	float 	fVec12[2048];
	int 	iConst61;
	float 	fRec50[2];
	float 	fConst62;
	float 	fConst63;
	float 	fConst64;
	float 	fRec57[2];
	float 	fConst65;
	float 	fRec56[2];
	float 	fVec13[16384];
	float 	fConst66;
	int 	iConst67;
	float 	fVec14[2048];
	int 	iConst68;
	float 	fRec54[2];
	float 	fConst69;
	float 	fConst70;
	float 	fConst71;
	float 	fRec61[2];
	float 	fConst72;
	float 	fRec60[2];
	float 	fVec15[16384];
	float 	fConst73;
	int 	iConst74;
	float 	fVec16[1024];
	int 	iConst75;
	float 	fRec58[2];
	float 	fRec0[3];
	float 	fRec1[3];
	float 	fRec2[3];
	float 	fRec3[3];
	float 	fRec4[3];
	float 	fRec5[3];
	float 	fRec6[3];
	float 	fRec7[3];
	float 	fVec17[4096];
	float 	fRec62[2];
	float 	fVec18[4096];
	float 	fRec63[2];
  public:
	static void metadata(Meta* m) 	{ 
		m->declare("name", "Modal Bar");
		m->declare("description", "Nonlinear Modal percussive instruments");
		m->declare("author", "Romain Michon (rmichon@ccrma.stanford.edu)");
		m->declare("copyright", "Romain Michon");
		m->declare("version", "1.0");
		m->declare("licence", "STK-4.3");
		m->declare("music.lib/name", "Music Library");
		m->declare("music.lib/author", "GRAME");
		m->declare("music.lib/copyright", "GRAME");
		m->declare("music.lib/version", "1.0");
		m->declare("music.lib/license", "LGPL with exception");
		m->declare("math.lib/name", "Math Library");
		m->declare("math.lib/author", "GRAME");
		m->declare("math.lib/copyright", "GRAME");
		m->declare("math.lib/version", "1.0");
		m->declare("math.lib/license", "LGPL with exception");
		m->declare("effect.lib/name", "Faust Audio Effect Library");
		m->declare("effect.lib/author", "Julius O. Smith (jos at ccrma.stanford.edu)");
		m->declare("effect.lib/copyright", "Julius O. Smith III");
		m->declare("effect.lib/version", "1.33");
		m->declare("effect.lib/license", "STK-4.3");
		m->declare("filter.lib/name", "Faust Filter Library");
		m->declare("filter.lib/author", "Julius O. Smith (jos at ccrma.stanford.edu)");
		m->declare("filter.lib/copyright", "Julius O. Smith III");
		m->declare("filter.lib/version", "1.29");
		m->declare("filter.lib/license", "STK-4.3");
		m->declare("filter.lib/reference", "https://ccrma.stanford.edu/~jos/filters/");
		m->declare("instrument.lib/name", "Faust-STK Tools Library");
		m->declare("instrument.lib/author", "Romain Michon (rmichon@ccrma.stanford.edu)");
		m->declare("instrument.lib/copyright", "Romain Michon");
		m->declare("instrument.lib/version", "1.0");
		m->declare("instrument.lib/licence", "STK-4.3");
	}

	virtual int getNumInputs() 	{ return 0; }
	virtual int getNumOutputs() 	{ return 2; }
	static void classInit(int samplingFreq) {
		SIG0 sig0;
		sig0.init(samplingFreq);
		sig0.fill(246,ftbl0);
	}
	virtual void instanceInit(int samplingFreq) {
		fSamplingFreq = samplingFreq;
		fslider0 = 0.5f;
		iConst0 = min(192000, max(1, fSamplingFreq));
		fConst1 = floorf((0.5f + (0.174713f * iConst0)));
		fConst2 = ((0 - (6.907755278982138f * fConst1)) / float(iConst0));
		fConst3 = (0.25f * fConst2);
		fConst4 = cosf((37699.11184307752f / float(iConst0)));
		fConst5 = (1.0f / tanf((628.3185307179587f / float(iConst0))));
		fConst6 = (1 + fConst5);
		fConst7 = (0 - ((1 - fConst5) / fConst6));
		fConst8 = (1.0f / fConst6);
		for (int i=0; i<2; i++) fRec11[i] = 0;
		fConst9 = (0.16666666666666666f * fConst2);
		for (int i=0; i<2; i++) fRec10[i] = 0;
		IOTA = 0;
		for (int i=0; i<8192; i++) fVec0[i] = 0;
		fConst10 = floorf((0.5f + (0.022904f * iConst0)));
		iConst11 = int((int((fConst1 - fConst10)) & 8191));
		fslider1 = 2.1f;
		fConst12 = (6.283185307179586f / float(iConst0));
		fslider2 = 0.99999f;
		for (int i=0; i<2; i++) fRec14[i] = 0;
		fConst13 = tanf((314.1592653589793f / float(iConst0)));
		fConst14 = (1.0f / faustpower<2>(fConst13));
		fConst15 = (2 * (1 - fConst14));
		fConst16 = (1.0f / fConst13);
		fConst17 = (1 + ((fConst16 - 1.414213562373095f) / fConst13));
		fConst18 = (1.0f / (1 + ((1.414213562373095f + fConst16) / fConst13)));
		fslider3 = 1e+01f;
		fbutton0 = 0.0;
		for (int i=0; i<3; i++) fRec16[i] = 0;
		fConst19 = (2 * (0 - fConst14));
		for (int i=0; i<2; i++) fRec15[i] = 0;
		fbutton1 = 0.0;
		fslider4 = 0.5f;
		for (int i=0; i<2; i++) fRec17[i] = 0;
		fConst20 = (0.0010000000000000009f * loadPreset(4, 2, 1));
		for (int i=0; i<2; i++) fRec18[i] = 0;
		for (int i=0; i<2; i++) iRec19[i] = 0;
		for (int i=0; i<2; i++) fRec20[i] = 0;
		for (int i=0; i<3; i++) fRec12[i] = 0;
		fConst21 = (0.0010000000000000009f * loadPreset(4, 2, 0));
		for (int i=0; i<2; i++) fRec22[i] = 0;
		for (int i=0; i<2; i++) fRec23[i] = 0;
		for (int i=0; i<3; i++) fRec21[i] = 0;
		fConst22 = (0.0010000000000000009f * loadPreset(4, 2, 2));
		for (int i=0; i<2; i++) fRec25[i] = 0;
		for (int i=0; i<2; i++) fRec26[i] = 0;
		for (int i=0; i<3; i++) fRec24[i] = 0;
		fConst23 = (0.0010000000000000009f * loadPreset(4, 2, 3));
		for (int i=0; i<2; i++) fRec28[i] = 0;
		for (int i=0; i<2; i++) fRec29[i] = 0;
		for (int i=0; i<3; i++) fRec27[i] = 0;
		for (int i=0; i<3; i++) fRec30[i] = 0;
		for (int i=0; i<3; i++) fRec31[i] = 0;
		for (int i=0; i<3; i++) fRec32[i] = 0;
		for (int i=0; i<3; i++) fRec33[i] = 0;
		fConst24 = loadPreset(4, 3, 2);
		for (int i=0; i<4096; i++) fVec1[i] = 0;
		iConst25 = int((int((0.02f * iConst0)) & 8191));
		for (int i=0; i<2048; i++) fVec2[i] = 0;
		iConst26 = int((int((fConst10 - 1)) & 2047));
		for (int i=0; i<2; i++) fRec8[i] = 0;
		fConst27 = floorf((0.5f + (0.153129f * iConst0)));
		fConst28 = ((0 - (6.907755278982138f * fConst27)) / float(iConst0));
		fConst29 = (0.25f * fConst28);
		for (int i=0; i<2; i++) fRec37[i] = 0;
		fConst30 = (0.16666666666666666f * fConst28);
		for (int i=0; i<2; i++) fRec36[i] = 0;
		for (int i=0; i<8192; i++) fVec3[i] = 0;
		fConst31 = floorf((0.5f + (0.020346f * iConst0)));
		iConst32 = int((int((fConst27 - fConst31)) & 8191));
		for (int i=0; i<1024; i++) fVec4[i] = 0;
		iConst33 = int((int((fConst31 - 1)) & 1023));
		for (int i=0; i<2; i++) fRec34[i] = 0;
		fConst34 = floorf((0.5f + (0.127837f * iConst0)));
		fConst35 = ((0 - (6.907755278982138f * fConst34)) / float(iConst0));
		fConst36 = (0.25f * fConst35);
		for (int i=0; i<2; i++) fRec41[i] = 0;
		fConst37 = (0.16666666666666666f * fConst35);
		for (int i=0; i<2; i++) fRec40[i] = 0;
		for (int i=0; i<8192; i++) fVec5[i] = 0;
		fConst38 = floorf((0.5f + (0.031604f * iConst0)));
		iConst39 = int((int((fConst34 - fConst38)) & 8191));
		for (int i=0; i<2048; i++) fVec6[i] = 0;
		iConst40 = int((int((fConst38 - 1)) & 2047));
		for (int i=0; i<2; i++) fRec38[i] = 0;
		fConst41 = floorf((0.5f + (0.125f * iConst0)));
		fConst42 = ((0 - (6.907755278982138f * fConst41)) / float(iConst0));
		fConst43 = (0.25f * fConst42);
		for (int i=0; i<2; i++) fRec45[i] = 0;
		fConst44 = (0.16666666666666666f * fConst42);
		for (int i=0; i<2; i++) fRec44[i] = 0;
		for (int i=0; i<8192; i++) fVec7[i] = 0;
		fConst45 = floorf((0.5f + (0.013458f * iConst0)));
		iConst46 = int((int((fConst41 - fConst45)) & 8191));
		for (int i=0; i<1024; i++) fVec8[i] = 0;
		iConst47 = int((int((fConst45 - 1)) & 1023));
		for (int i=0; i<2; i++) fRec42[i] = 0;
		fConst48 = floorf((0.5f + (0.210389f * iConst0)));
		fConst49 = ((0 - (6.907755278982138f * fConst48)) / float(iConst0));
		fConst50 = (0.25f * fConst49);
		for (int i=0; i<2; i++) fRec49[i] = 0;
		fConst51 = (0.16666666666666666f * fConst49);
		for (int i=0; i<2; i++) fRec48[i] = 0;
		for (int i=0; i<16384; i++) fVec9[i] = 0;
		fConst52 = floorf((0.5f + (0.024421f * iConst0)));
		iConst53 = int((int((fConst48 - fConst52)) & 16383));
		for (int i=0; i<2048; i++) fVec10[i] = 0;
		iConst54 = int((int((fConst52 - 1)) & 2047));
		for (int i=0; i<2; i++) fRec46[i] = 0;
		fConst55 = floorf((0.5f + (0.192303f * iConst0)));
		fConst56 = ((0 - (6.907755278982138f * fConst55)) / float(iConst0));
		fConst57 = (0.25f * fConst56);
		for (int i=0; i<2; i++) fRec53[i] = 0;
		fConst58 = (0.16666666666666666f * fConst56);
		for (int i=0; i<2; i++) fRec52[i] = 0;
		for (int i=0; i<8192; i++) fVec11[i] = 0;
		fConst59 = floorf((0.5f + (0.029291f * iConst0)));
		iConst60 = int((int((fConst55 - fConst59)) & 8191));
		for (int i=0; i<2048; i++) fVec12[i] = 0;
		iConst61 = int((int((fConst59 - 1)) & 2047));
		for (int i=0; i<2; i++) fRec50[i] = 0;
		fConst62 = floorf((0.5f + (0.256891f * iConst0)));
		fConst63 = ((0 - (6.907755278982138f * fConst62)) / float(iConst0));
		fConst64 = (0.25f * fConst63);
		for (int i=0; i<2; i++) fRec57[i] = 0;
		fConst65 = (0.16666666666666666f * fConst63);
		for (int i=0; i<2; i++) fRec56[i] = 0;
		for (int i=0; i<16384; i++) fVec13[i] = 0;
		fConst66 = floorf((0.5f + (0.027333f * iConst0)));
		iConst67 = int((int((fConst62 - fConst66)) & 16383));
		for (int i=0; i<2048; i++) fVec14[i] = 0;
		iConst68 = int((int((fConst66 - 1)) & 2047));
		for (int i=0; i<2; i++) fRec54[i] = 0;
		fConst69 = floorf((0.5f + (0.219991f * iConst0)));
		fConst70 = ((0 - (6.907755278982138f * fConst69)) / float(iConst0));
		fConst71 = (0.25f * fConst70);
		for (int i=0; i<2; i++) fRec61[i] = 0;
		fConst72 = (0.16666666666666666f * fConst70);
		for (int i=0; i<2; i++) fRec60[i] = 0;
		for (int i=0; i<16384; i++) fVec15[i] = 0;
		fConst73 = floorf((0.5f + (0.019123f * iConst0)));
		iConst74 = int((int((fConst69 - fConst73)) & 16383));
		for (int i=0; i<1024; i++) fVec16[i] = 0;
		iConst75 = int((int((fConst73 - 1)) & 1023));
		for (int i=0; i<2; i++) fRec58[i] = 0;
		for (int i=0; i<3; i++) fRec0[i] = 0;
		for (int i=0; i<3; i++) fRec1[i] = 0;
		for (int i=0; i<3; i++) fRec2[i] = 0;
		for (int i=0; i<3; i++) fRec3[i] = 0;
		for (int i=0; i<3; i++) fRec4[i] = 0;
		for (int i=0; i<3; i++) fRec5[i] = 0;
		for (int i=0; i<3; i++) fRec6[i] = 0;
		for (int i=0; i<3; i++) fRec7[i] = 0;
		for (int i=0; i<4096; i++) fVec17[i] = 0;
		for (int i=0; i<2; i++) fRec62[i] = 0;
		for (int i=0; i<4096; i++) fVec18[i] = 0;
		for (int i=0; i<2; i++) fRec63[i] = 0;
	}
	virtual void init(int samplingFreq) {
		classInit(samplingFreq);
		instanceInit(samplingFreq);
	}
	virtual void buildUserInterface(UI* interface) {
		interface->openVerticalBox("modalBarCC3");
		interface->openHorizontalBox("Basic_Parameters");
		interface->declare(&fbutton1, "1", "");
		interface->declare(&fbutton1, "tooltip", "noteOn = 1, noteOff = 0");
		interface->addButton("hit", &fbutton1);
		interface->closeBox();
		interface->addHorizontalSlider("amp", &fslider3, 1e+01f, 5.0f, 2e+02f, 1.0f);
		interface->addHorizontalSlider("depth", &fslider0, 0.5f, 0.01f, 1.0f, 0.001f);
		interface->addHorizontalSlider("fre", &fslider1, 2.1f, 0.0f, 1e+01f, 0.001f);
		interface->addButton("poke", &fbutton0);
		interface->addHorizontalSlider("que", &fslider2, 0.99999f, 0.9999f, 1.0f, 1e-05f);
		interface->addHorizontalSlider("users", &fslider4, 0.5f, 0.0f, 1.0f, 0.0001f);
		interface->closeBox();
	}
	virtual void compute (int count, FAUSTFLOAT** input, FAUSTFLOAT** output) {
		float 	fSlow0 = fslider0;
		float 	fSlow1 = expf((fConst3 / fSlow0));
		float 	fSlow2 = faustpower<2>(fSlow1);
		float 	fSlow3 = (1.0f - fSlow2);
		float 	fSlow4 = (1.0f - (fConst4 * fSlow2));
		float 	fSlow5 = sqrtf(max((float)0, ((faustpower<2>(fSlow4) / faustpower<2>(fSlow3)) - 1.0f)));
		float 	fSlow6 = (fSlow4 / fSlow3);
		float 	fSlow7 = (fSlow6 - fSlow5);
		float 	fSlow8 = ((expf((fConst9 / fSlow0)) / fSlow1) - 1);
		float 	fSlow9 = (fSlow1 * ((1.0f + fSlow5) - fSlow6));
		float 	fSlow10 = (fConst12 * fslider1);
		float 	fSlow11 = sinf(fSlow10);
		float 	fSlow12 = cosf(fSlow10);
		float 	fSlow13 = fslider2;
		float 	fSlow14 = (0 - fSlow11);
		float 	fSlow15 = (fbutton0 * fslider3);
		float 	fSlow16 = fbutton1;
		float 	fSlow17 = fslider4;
		float 	fSlow18 = (0.05f + (0.9f * (1.0f - fSlow17)));
		float 	fSlow19 = (246 * fSlow18);
		float 	fSlow20 = (1500 * fSlow17);
		float 	fSlow21 = (1.4e+02f + fSlow20);
		int 	iSlow22 = (fSlow0 > 0.75f);
		float 	fSlow23 = (7e+02f + fSlow20);
		float 	fSlow24 = (30 * fSlow0);
		float 	fSlow25 = expf((fConst29 / fSlow0));
		float 	fSlow26 = faustpower<2>(fSlow25);
		float 	fSlow27 = (1.0f - fSlow26);
		float 	fSlow28 = (1.0f - (fConst4 * fSlow26));
		float 	fSlow29 = sqrtf(max((float)0, ((faustpower<2>(fSlow28) / faustpower<2>(fSlow27)) - 1.0f)));
		float 	fSlow30 = (fSlow28 / fSlow27);
		float 	fSlow31 = (fSlow30 - fSlow29);
		float 	fSlow32 = ((expf((fConst30 / fSlow0)) / fSlow25) - 1);
		float 	fSlow33 = (fSlow25 * ((1.0f + fSlow29) - fSlow30));
		float 	fSlow34 = expf((fConst36 / fSlow0));
		float 	fSlow35 = faustpower<2>(fSlow34);
		float 	fSlow36 = (1.0f - fSlow35);
		float 	fSlow37 = (1.0f - (fConst4 * fSlow35));
		float 	fSlow38 = sqrtf(max((float)0, ((faustpower<2>(fSlow37) / faustpower<2>(fSlow36)) - 1.0f)));
		float 	fSlow39 = (fSlow37 / fSlow36);
		float 	fSlow40 = (fSlow39 - fSlow38);
		float 	fSlow41 = ((expf((fConst37 / fSlow0)) / fSlow34) - 1);
		float 	fSlow42 = (fSlow34 * ((1.0f + fSlow38) - fSlow39));
		float 	fSlow43 = expf((fConst43 / fSlow0));
		float 	fSlow44 = faustpower<2>(fSlow43);
		float 	fSlow45 = (1.0f - fSlow44);
		float 	fSlow46 = (1.0f - (fConst4 * fSlow44));
		float 	fSlow47 = sqrtf(max((float)0, ((faustpower<2>(fSlow46) / faustpower<2>(fSlow45)) - 1.0f)));
		float 	fSlow48 = (fSlow46 / fSlow45);
		float 	fSlow49 = (fSlow48 - fSlow47);
		float 	fSlow50 = ((expf((fConst44 / fSlow0)) / fSlow43) - 1);
		float 	fSlow51 = (fSlow43 * ((1.0f + fSlow47) - fSlow48));
		float 	fSlow52 = expf((fConst50 / fSlow0));
		float 	fSlow53 = faustpower<2>(fSlow52);
		float 	fSlow54 = (1.0f - fSlow53);
		float 	fSlow55 = (1.0f - (fConst4 * fSlow53));
		float 	fSlow56 = sqrtf(max((float)0, ((faustpower<2>(fSlow55) / faustpower<2>(fSlow54)) - 1.0f)));
		float 	fSlow57 = (fSlow55 / fSlow54);
		float 	fSlow58 = (fSlow57 - fSlow56);
		float 	fSlow59 = ((expf((fConst51 / fSlow0)) / fSlow52) - 1);
		float 	fSlow60 = (fSlow52 * ((1.0f + fSlow56) - fSlow57));
		float 	fSlow61 = expf((fConst57 / fSlow0));
		float 	fSlow62 = faustpower<2>(fSlow61);
		float 	fSlow63 = (1.0f - fSlow62);
		float 	fSlow64 = (1.0f - (fConst4 * fSlow62));
		float 	fSlow65 = sqrtf(max((float)0, ((faustpower<2>(fSlow64) / faustpower<2>(fSlow63)) - 1.0f)));
		float 	fSlow66 = (fSlow64 / fSlow63);
		float 	fSlow67 = (fSlow66 - fSlow65);
		float 	fSlow68 = ((expf((fConst58 / fSlow0)) / fSlow61) - 1);
		float 	fSlow69 = (fSlow61 * ((1.0f + fSlow65) - fSlow66));
		float 	fSlow70 = expf((fConst64 / fSlow0));
		float 	fSlow71 = faustpower<2>(fSlow70);
		float 	fSlow72 = (1.0f - fSlow71);
		float 	fSlow73 = (1.0f - (fConst4 * fSlow71));
		float 	fSlow74 = sqrtf(max((float)0, ((faustpower<2>(fSlow73) / faustpower<2>(fSlow72)) - 1.0f)));
		float 	fSlow75 = (fSlow73 / fSlow72);
		float 	fSlow76 = (fSlow75 - fSlow74);
		float 	fSlow77 = ((expf((fConst65 / fSlow0)) / fSlow70) - 1);
		float 	fSlow78 = (fSlow70 * ((1.0f + fSlow74) - fSlow75));
		float 	fSlow79 = expf((fConst71 / fSlow0));
		float 	fSlow80 = faustpower<2>(fSlow79);
		float 	fSlow81 = (1.0f - fSlow80);
		float 	fSlow82 = (1.0f - (fConst4 * fSlow80));
		float 	fSlow83 = sqrtf(max((float)0, ((faustpower<2>(fSlow82) / faustpower<2>(fSlow81)) - 1.0f)));
		float 	fSlow84 = (fSlow82 / fSlow81);
		float 	fSlow85 = (fSlow84 - fSlow83);
		float 	fSlow86 = ((expf((fConst72 / fSlow0)) / fSlow79) - 1);
		float 	fSlow87 = (fSlow79 * ((1.0f + fSlow83) - fSlow84));
		float 	fSlow88 = (30 * (1 - fSlow0));
		float 	fSlow89 = (0.6f * fSlow0);
		float 	fSlow90 = (1.5f * (0.9f - fSlow89));
		float 	fSlow91 = (0.93f + (0.05f * fSlow0));
		float 	fSlow92 = (1.5f * (0.1f + fSlow89));
		float 	fSlow93 = (3000 * fSlow0);
		float 	fSlow94 = (300 + fSlow93);
		int 	iSlow95 = int(fSlow94);
		int 	iSlow96 = int((iSlow95 & 4095));
		float 	fSlow97 = (iSlow95 - (299 + fSlow93));
		int 	iSlow98 = int((int((1 + iSlow95)) & 4095));
		float 	fSlow99 = (fSlow94 - iSlow95);
		FAUSTFLOAT* output0 = output[0];
		FAUSTFLOAT* output1 = output[1];
		for (int i=0; i<count; i++) {
			fRec11[0] = ((fConst8 * (fRec4[1] + fRec4[2])) + (fConst7 * fRec11[1]));
			fRec10[0] = ((fSlow9 * (fRec4[1] + (fSlow8 * fRec11[0]))) + (fSlow7 * fRec10[1]));
			fVec0[IOTA&8191] = (1e-20f + (0.35355339059327373f * fRec10[0]));
			fRec14[0] = (fSlow13 * ((fSlow12 * fRec14[1]) + (fSlow11 * fRec15[1])));
			fRec16[0] = (fSlow15 - (fConst18 * ((fConst17 * fRec16[2]) + (fConst15 * fRec16[1]))));
			fRec15[0] = ((fSlow12 * fRec15[1]) + ((fConst18 * (((fConst14 * fRec16[0]) + (fConst19 * fRec16[1])) + (fConst14 * fRec16[2]))) + (fSlow14 * fRec14[1])));
			int iTemp0 = ((int(((fSlow16 + fRec14[0]) >= 1)))?1:0);
			float fTemp1 = (fSlow18 + fRec17[1]);
			fRec17[0] = (fTemp1 - floorf(fTemp1));
			float fTemp2 = ftbl0[int((246.0f * (fRec17[0] * iTemp0)))];
			fRec18[0] = (fConst20 + (0.999f * fRec18[1]));
			iRec19[0] = (1 + (iTemp0 * iRec19[1]));
			int iTemp3 = ((iRec19[0] - 1) < fSlow19);
			float fTemp4 = (0.001f * (((iTemp3 * fRec18[0]) * iTemp0) * fTemp2));
			fRec20[0] = (0.0010000000000000009f + (0.999f * fRec20[1]));
			float fTemp5 = loadPreset(4, 0, fRec20[0]);
			int iTemp6 = (fTemp5 < 0);
			float fTemp7 = (fTemp5 * (0 - (iTemp6 - 1)));
			float fTemp8 = (iTemp6 * (0 - fTemp5));
			float fTemp9 = (1 - (0.024f * ((iTemp0 < 1) & iSlow22)));
			float fTemp10 = loadPreset(4, 1, fRec20[0]);
			float fTemp11 = (0 - (2 * (fTemp10 * fTemp9)));
			float fTemp12 = faustpower<2>(fTemp9);
			float fTemp13 = (faustpower<2>(fTemp10) * fTemp12);
			fRec12[0] = (0 - (((fTemp13 * fRec12[2]) + ((fTemp11 * fRec12[1]) * cosf((fConst12 * (fTemp8 + (fSlow21 * fTemp7)))))) - fTemp4));
			fRec22[0] = (fConst21 + (0.999f * fRec22[1]));
			float fTemp14 = (0.001f * (((iTemp3 * fRec22[0]) * iTemp0) * fTemp2));
			fRec23[0] = (0.999f * fRec23[1]);
			float fTemp15 = loadPreset(4, 0, fRec23[0]);
			int iTemp16 = (fTemp15 < 0);
			float fTemp17 = (fTemp15 * (0 - (iTemp16 - 1)));
			float fTemp18 = (iTemp16 * (0 - fTemp15));
			float fTemp19 = loadPreset(4, 1, fRec23[0]);
			float fTemp20 = (0 - (2 * (fTemp19 * fTemp9)));
			float fTemp21 = (faustpower<2>(fTemp19) * fTemp12);
			fRec21[0] = (0 - (((fTemp21 * fRec21[2]) + ((fTemp20 * fRec21[1]) * cosf((fConst12 * (fTemp18 + (fSlow21 * fTemp17)))))) - fTemp14));
			fRec25[0] = (fConst22 + (0.999f * fRec25[1]));
			float fTemp22 = (0.001f * (((iTemp3 * fRec25[0]) * iTemp0) * fTemp2));
			fRec26[0] = (0.0020000000000000018f + (0.999f * fRec26[1]));
			float fTemp23 = loadPreset(4, 0, fRec26[0]);
			int iTemp24 = (fTemp23 < 0);
			float fTemp25 = (fTemp23 * (0 - (iTemp24 - 1)));
			float fTemp26 = (iTemp24 * (0 - fTemp23));
			float fTemp27 = loadPreset(4, 1, fRec26[0]);
			float fTemp28 = (0 - (2 * (fTemp27 * fTemp9)));
			float fTemp29 = (faustpower<2>(fTemp27) * fTemp12);
			fRec24[0] = (0 - (((fTemp29 * fRec24[2]) + ((fTemp28 * fRec24[1]) * cosf((fConst12 * (fTemp26 + (fSlow21 * fTemp25)))))) - fTemp22));
			fRec28[0] = (fConst23 + (0.999f * fRec28[1]));
			float fTemp30 = (0.001f * (((iTemp3 * fRec28[0]) * iTemp0) * fTemp2));
			fRec29[0] = (0.0030000000000000027f + (0.999f * fRec29[1]));
			float fTemp31 = loadPreset(4, 0, fRec29[0]);
			int iTemp32 = (fTemp31 < 0);
			float fTemp33 = (fTemp31 * (0 - (iTemp32 - 1)));
			float fTemp34 = (iTemp32 * (0 - fTemp31));
			float fTemp35 = loadPreset(4, 1, fRec29[0]);
			float fTemp36 = (0 - (2 * (fTemp35 * fTemp9)));
			float fTemp37 = faustpower<2>(fTemp35);
			fRec27[0] = (0 - ((((fTemp37 * fTemp12) * fRec27[2]) + ((fTemp36 * fRec27[1]) * cosf((fConst12 * (fTemp34 + (fSlow21 * fTemp33)))))) - fTemp30));
			float fTemp38 = (fRec27[0] + (fRec24[0] + (fRec21[0] + fRec12[0])));
			fRec30[0] = (0 - (((fTemp21 * fRec30[2]) + ((fRec30[1] * cosf((fConst12 * (fTemp18 + (fSlow23 * fTemp17))))) * fTemp20)) - fTemp14));
			fRec31[0] = (0 - (((fTemp13 * fRec31[2]) + ((fRec31[1] * cosf((fConst12 * (fTemp8 + (fSlow23 * fTemp7))))) * fTemp11)) - fTemp4));
			fRec32[0] = (0 - (((fTemp29 * fRec32[2]) + ((fRec32[1] * cosf((fConst12 * (fTemp26 + (fSlow23 * fTemp25))))) * fTemp28)) - fTemp22));
			fRec33[0] = (0 - ((((fTemp37 * fRec33[2]) * fTemp12) + ((fRec33[1] * cosf((fConst12 * (fTemp34 + (fSlow23 * fTemp33))))) * fTemp36)) - fTemp30));
			float fTemp39 = (0.001f * ((iTemp3 * iTemp0) * fTemp2));
			float fTemp40 = ((fConst24 * ((fTemp39 - (fRec33[0] + (fRec32[0] + (fRec30[0] + fRec31[0])))) + (fTemp39 - fTemp38))) + (fRec33[0] + (fRec32[0] + (fRec31[0] + (fRec30[0] + fTemp38)))));
			fVec1[IOTA&4095] = (fSlow24 * fTemp40);
			float fTemp41 = (0.3f * fVec1[(IOTA-iConst25)&4095]);
			float fTemp42 = ((fTemp41 + fVec0[(IOTA-iConst11)&8191]) - (0.6f * fRec8[1]));
			fVec2[IOTA&2047] = fTemp42;
			fRec8[0] = fVec2[(IOTA-iConst26)&2047];
			float 	fRec9 = (0.6f * fVec2[IOTA&2047]);
			fRec37[0] = ((fConst8 * (fRec0[1] + fRec0[2])) + (fConst7 * fRec37[1]));
			fRec36[0] = ((fSlow33 * (fRec0[1] + (fSlow32 * fRec37[0]))) + (fSlow31 * fRec36[1]));
			fVec3[IOTA&8191] = (1e-20f + (0.35355339059327373f * fRec36[0]));
			float fTemp43 = ((fTemp41 + fVec3[(IOTA-iConst32)&8191]) - (0.6f * fRec34[1]));
			fVec4[IOTA&1023] = fTemp43;
			fRec34[0] = fVec4[(IOTA-iConst33)&1023];
			float 	fRec35 = (0.6f * fVec4[IOTA&1023]);
			float fTemp44 = (fRec35 + fRec9);
			fRec41[0] = ((fConst8 * (fRec2[1] + fRec2[2])) + (fConst7 * fRec41[1]));
			fRec40[0] = ((fSlow42 * (fRec2[1] + (fSlow41 * fRec41[0]))) + (fSlow40 * fRec40[1]));
			fVec5[IOTA&8191] = (1e-20f + (0.35355339059327373f * fRec40[0]));
			float fTemp45 = (fVec5[(IOTA-iConst39)&8191] - (fTemp41 + (0.6f * fRec38[1])));
			fVec6[IOTA&2047] = fTemp45;
			fRec38[0] = fVec6[(IOTA-iConst40)&2047];
			float 	fRec39 = (0.6f * fVec6[IOTA&2047]);
			fRec45[0] = ((fConst8 * (fRec6[1] + fRec6[2])) + (fConst7 * fRec45[1]));
			fRec44[0] = ((fSlow51 * (fRec6[1] + (fSlow50 * fRec45[0]))) + (fSlow49 * fRec44[1]));
			fVec7[IOTA&8191] = (1e-20f + (0.35355339059327373f * fRec44[0]));
			float fTemp46 = (fVec7[(IOTA-iConst46)&8191] - (fTemp41 + (0.6f * fRec42[1])));
			fVec8[IOTA&1023] = fTemp46;
			fRec42[0] = fVec8[(IOTA-iConst47)&1023];
			float 	fRec43 = (0.6f * fVec8[IOTA&1023]);
			float fTemp47 = (fRec43 + (fRec39 + fTemp44));
			fRec49[0] = ((fConst8 * (fRec1[1] + fRec1[2])) + (fConst7 * fRec49[1]));
			fRec48[0] = ((fSlow60 * (fRec1[1] + (fSlow59 * fRec49[0]))) + (fSlow58 * fRec48[1]));
			fVec9[IOTA&16383] = (1e-20f + (0.35355339059327373f * fRec48[0]));
			float fTemp48 = ((fTemp41 + fVec9[(IOTA-iConst53)&16383]) + (0.6f * fRec46[1]));
			fVec10[IOTA&2047] = fTemp48;
			fRec46[0] = fVec10[(IOTA-iConst54)&2047];
			float 	fRec47 = (0 - (0.6f * fVec10[IOTA&2047]));
			fRec53[0] = ((fConst8 * (fRec5[1] + fRec5[2])) + (fConst7 * fRec53[1]));
			fRec52[0] = ((fSlow69 * (fRec5[1] + (fSlow68 * fRec53[0]))) + (fSlow67 * fRec52[1]));
			fVec11[IOTA&8191] = (1e-20f + (0.35355339059327373f * fRec52[0]));
			float fTemp49 = ((fTemp41 + fVec11[(IOTA-iConst60)&8191]) + (0.6f * fRec50[1]));
			fVec12[IOTA&2047] = fTemp49;
			fRec50[0] = fVec12[(IOTA-iConst61)&2047];
			float 	fRec51 = (0 - (0.6f * fVec12[IOTA&2047]));
			fRec57[0] = ((fConst8 * (fRec3[1] + fRec3[2])) + (fConst7 * fRec57[1]));
			fRec56[0] = ((fSlow78 * (fRec3[1] + (fSlow77 * fRec57[0]))) + (fSlow76 * fRec56[1]));
			fVec13[IOTA&16383] = (1e-20f + (0.35355339059327373f * fRec56[0]));
			float fTemp50 = ((fVec13[(IOTA-iConst67)&16383] + (0.6f * fRec54[1])) - fTemp41);
			fVec14[IOTA&2047] = fTemp50;
			fRec54[0] = fVec14[(IOTA-iConst68)&2047];
			float 	fRec55 = (0 - (0.6f * fVec14[IOTA&2047]));
			fRec61[0] = ((fConst8 * (fRec7[1] + fRec7[2])) + (fConst7 * fRec61[1]));
			fRec60[0] = ((fSlow87 * (fRec7[1] + (fSlow86 * fRec61[0]))) + (fSlow85 * fRec60[1]));
			fVec15[IOTA&16383] = (1e-20f + (0.35355339059327373f * fRec60[0]));
			float fTemp51 = ((fVec15[(IOTA-iConst74)&16383] + (0.6f * fRec58[1])) - fTemp41);
			fVec16[IOTA&1023] = fTemp51;
			fRec58[0] = fVec16[(IOTA-iConst75)&1023];
			float 	fRec59 = (0 - (0.6f * fVec16[IOTA&1023]));
			fRec0[0] = (fRec34[1] + (fRec8[1] + (fRec38[1] + (fRec42[1] + (fRec46[1] + (fRec50[1] + (fRec54[1] + (fRec58[1] + (fRec59 + (fRec55 + (fRec51 + (fRec47 + fTemp47))))))))))));
			fRec1[0] = (0 - ((fRec46[1] + (fRec50[1] + (fRec54[1] + (fRec58[1] + (fRec59 + (fRec55 + (fRec47 + fRec51))))))) - (fRec34[1] + (fRec8[1] + (fRec38[1] + (fRec42[1] + fTemp47))))));
			float fTemp52 = (fRec39 + fRec43);
			fRec2[0] = (0 - ((fRec38[1] + (fRec42[1] + (fRec54[1] + (fRec58[1] + (fRec59 + (fRec55 + fTemp52)))))) - (fRec34[1] + (fRec8[1] + (fRec46[1] + (fRec50[1] + (fRec51 + (fRec47 + fTemp44))))))));
			fRec3[0] = (0 - ((fRec38[1] + (fRec42[1] + (fRec46[1] + (fRec50[1] + (fRec51 + (fRec47 + fTemp52)))))) - (fRec34[1] + (fRec8[1] + (fRec54[1] + (fRec58[1] + (fRec59 + (fRec55 + fTemp44))))))));
			float fTemp53 = (fRec35 + fRec39);
			float fTemp54 = (fRec9 + fRec43);
			fRec4[0] = (0 - ((fRec8[1] + (fRec42[1] + (fRec50[1] + (fRec58[1] + (fRec59 + (fRec51 + fTemp54)))))) - (fRec34[1] + (fRec38[1] + (fRec46[1] + (fRec54[1] + (fRec55 + (fRec47 + fTemp53))))))));
			fRec5[0] = (0 - ((fRec8[1] + (fRec42[1] + (fRec46[1] + (fRec54[1] + (fRec55 + (fRec47 + fTemp54)))))) - (fRec34[1] + (fRec38[1] + (fRec50[1] + (fRec58[1] + (fRec59 + (fRec51 + fTemp53))))))));
			float fTemp55 = (fRec35 + fRec43);
			float fTemp56 = (fRec9 + fRec39);
			fRec6[0] = (0 - ((fRec8[1] + (fRec38[1] + (fRec50[1] + (fRec54[1] + (fRec55 + (fRec51 + fTemp56)))))) - (fRec34[1] + (fRec42[1] + (fRec46[1] + (fRec58[1] + (fRec59 + (fRec47 + fTemp55))))))));
			fRec7[0] = (0 - ((fRec8[1] + (fRec38[1] + (fRec46[1] + (fRec58[1] + (fRec59 + (fRec47 + fTemp56)))))) - (fRec34[1] + (fRec42[1] + (fRec50[1] + (fRec54[1] + (fRec55 + (fRec51 + fTemp55))))))));
			float fTemp57 = (fSlow88 * fTemp40);
			float fTemp58 = (fTemp57 + (0.37f * (fRec1[0] + fRec2[0])));
			float fTemp59 = ((fSlow92 * fTemp58) + (fSlow91 * fRec62[1]));
			fVec17[IOTA&4095] = fTemp59;
			fRec62[0] = ((fSlow99 * fVec17[(IOTA-iSlow98)&4095]) + (fSlow97 * fVec17[(IOTA-iSlow96)&4095]));
			output0[i] = (FAUSTFLOAT)(fRec62[0] + (fSlow90 * fTemp58));
			float fTemp60 = (fTemp57 + (0.37f * (fRec1[0] - fRec2[0])));
			float fTemp61 = ((fSlow92 * fTemp60) + (fSlow91 * fRec63[1]));
			fVec18[IOTA&4095] = fTemp61;
			fRec63[0] = ((fSlow99 * fVec18[(IOTA-iSlow98)&4095]) + (fSlow97 * fVec18[(IOTA-iSlow96)&4095]));
			output1[i] = (FAUSTFLOAT)(fRec63[0] + (fSlow90 * fTemp60));
			// post processing
			fRec63[1] = fRec63[0];
			fRec62[1] = fRec62[0];
			fRec7[2] = fRec7[1]; fRec7[1] = fRec7[0];
			fRec6[2] = fRec6[1]; fRec6[1] = fRec6[0];
			fRec5[2] = fRec5[1]; fRec5[1] = fRec5[0];
			fRec4[2] = fRec4[1]; fRec4[1] = fRec4[0];
			fRec3[2] = fRec3[1]; fRec3[1] = fRec3[0];
			fRec2[2] = fRec2[1]; fRec2[1] = fRec2[0];
			fRec1[2] = fRec1[1]; fRec1[1] = fRec1[0];
			fRec0[2] = fRec0[1]; fRec0[1] = fRec0[0];
			fRec58[1] = fRec58[0];
			fRec60[1] = fRec60[0];
			fRec61[1] = fRec61[0];
			fRec54[1] = fRec54[0];
			fRec56[1] = fRec56[0];
			fRec57[1] = fRec57[0];
			fRec50[1] = fRec50[0];
			fRec52[1] = fRec52[0];
			fRec53[1] = fRec53[0];
			fRec46[1] = fRec46[0];
			fRec48[1] = fRec48[0];
			fRec49[1] = fRec49[0];
			fRec42[1] = fRec42[0];
			fRec44[1] = fRec44[0];
			fRec45[1] = fRec45[0];
			fRec38[1] = fRec38[0];
			fRec40[1] = fRec40[0];
			fRec41[1] = fRec41[0];
			fRec34[1] = fRec34[0];
			fRec36[1] = fRec36[0];
			fRec37[1] = fRec37[0];
			fRec8[1] = fRec8[0];
			fRec33[2] = fRec33[1]; fRec33[1] = fRec33[0];
			fRec32[2] = fRec32[1]; fRec32[1] = fRec32[0];
			fRec31[2] = fRec31[1]; fRec31[1] = fRec31[0];
			fRec30[2] = fRec30[1]; fRec30[1] = fRec30[0];
			fRec27[2] = fRec27[1]; fRec27[1] = fRec27[0];
			fRec29[1] = fRec29[0];
			fRec28[1] = fRec28[0];
			fRec24[2] = fRec24[1]; fRec24[1] = fRec24[0];
			fRec26[1] = fRec26[0];
			fRec25[1] = fRec25[0];
			fRec21[2] = fRec21[1]; fRec21[1] = fRec21[0];
			fRec23[1] = fRec23[0];
			fRec22[1] = fRec22[0];
			fRec12[2] = fRec12[1]; fRec12[1] = fRec12[0];
			fRec20[1] = fRec20[0];
			iRec19[1] = iRec19[0];
			fRec18[1] = fRec18[0];
			fRec17[1] = fRec17[0];
			fRec15[1] = fRec15[0];
			fRec16[2] = fRec16[1]; fRec16[1] = fRec16[0];
			fRec14[1] = fRec14[0];
			IOTA = IOTA+1;
			fRec10[1] = fRec10[0];
			fRec11[1] = fRec11[0];
		}
	}
};


float 	mydsp::ftbl0[246];

#include <stdio.h>
#include <string.h>
#include "/usr/include/pdextended/m_pd.h"

#define faust_setup(name) xfaust_setup(name)
#define xfaust_setup(name) name ## _tilde_setup(void)
#define sym(name) xsym(name)
#define xsym(name) #name

// time for "active" toggle xfades in secs
#define XFADE_TIME 0.1f

static t_class *faust_class;

struct t_faust {
  t_object x_obj;
#ifdef __MINGW32__
  /* This seems to be necessary as some as yet undetermined Pd routine seems
     to write past the end of x_obj on Windows. */
  int fence; /* dummy field (not used) */
#endif
  mydsp *dsp;
  PdUI *ui;
  string *label;
  int active, xfade, n_xfade, rate, n_in, n_out;
  t_sample **inputs, **outputs, **buf;
  t_outlet *out;
  t_sample f;
};

static t_symbol *s_button, *s_checkbox, *s_vslider, *s_hslider, *s_nentry,
  *s_vbargraph, *s_hbargraph;

static inline void zero_samples(int k, int n, t_sample **out)
{
  for (int i = 0; i < k; i++)
#ifdef __STDC_IEC_559__
    /* IEC 559 a.k.a. IEEE 754 floats can be initialized faster like this */
    memset(out[i], 0, n*sizeof(t_sample));
#else
    for (int j = 0; j < n; j++)
      out[i][j] = 0.0f;
#endif
}

static inline void copy_samples(int k, int n, t_sample **out, t_sample **in)
{
  for (int i = 0; i < k; i++)
    memcpy(out[i], in[i], n*sizeof(t_sample));
}

static t_int *faust_perform(t_int *w)
{
  t_faust *x = (t_faust *)(w[1]);
  int n = (int)(w[2]);
  if (!x->dsp || !x->buf) return (w+3);
  AVOIDDENORMALS;
  if (x->xfade > 0) {
    float d = 1.0f/x->n_xfade, f = (x->xfade--)*d;
    d = d/n;
    x->dsp->compute(n, x->inputs, x->buf);
    if (x->active)
      if (x->n_in == x->n_out)
	/* xfade inputs -> buf */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->inputs[i][j]+(1.0f-f)*x->buf[i][j];
      else
	/* xfade 0 -> buf */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = (1.0f-f)*x->buf[i][j];
    else
      if (x->n_in == x->n_out)
	/* xfade buf -> inputs */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->buf[i][j]+(1.0f-f)*x->inputs[i][j];
      else
	/* xfade buf -> 0 */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->buf[i][j];
  } else if (x->active) {
    x->dsp->compute(n, x->inputs, x->buf);
    copy_samples(x->n_out, n, x->outputs, x->buf);
  } else if (x->n_in == x->n_out) {
    copy_samples(x->n_out, n, x->buf, x->inputs);
    copy_samples(x->n_out, n, x->outputs, x->buf);
  } else
    zero_samples(x->n_out, n, x->outputs);
  return (w+3);
}

static void faust_dsp(t_faust *x, t_signal **sp)
{
  int n = sp[0]->s_n, sr = (int)sp[0]->s_sr;
  if (x->rate <= 0) {
    /* default sample rate is whatever Pd tells us */
    PdUI *ui = x->ui;
    float *z = NULL;
    if (ui->nelems > 0 &&
	(z = (float*)malloc(ui->nelems*sizeof(float)))) {
      /* save the current control values */
      for (int i = 0; i < ui->nelems; i++)
	if (ui->elems[i].zone)
	  z[i] = *ui->elems[i].zone;
    }
    /* set the proper sample rate; this requires reinitializing the dsp */
    x->rate = sr;
    x->dsp->init(sr);
    if (z) {
      /* restore previous control values */
      for (int i = 0; i < ui->nelems; i++)
	if (ui->elems[i].zone)
	  *ui->elems[i].zone = z[i];
      free(z);
    }
  }
  if (n > 0)
    x->n_xfade = (int)(x->rate*XFADE_TIME/n);
  dsp_add(faust_perform, 2, x, n);
  for (int i = 0; i < x->n_in; i++)
    x->inputs[i] = sp[i+1]->s_vec;
  for (int i = 0; i < x->n_out; i++)
    x->outputs[i] = sp[x->n_in+i+1]->s_vec;
  if (x->buf != NULL)
    for (int i = 0; i < x->n_out; i++) {
      x->buf[i] = (t_sample*)malloc(n*sizeof(t_sample));
      if (x->buf[i] == NULL) {
	for (int j = 0; j < i; j++)
	  free(x->buf[j]);
	free(x->buf);
	x->buf = NULL;
	break;
      }
    }
}

static int pathcmp(const char *s, const char *t)
{
  int n = strlen(s), m = strlen(t);
  if (n == 0 || m == 0)
    return 0;
  else if (t[0] == '/')
    return strcmp(s, t);
  else if (n <= m || s[n-m-1] != '/')
    return strcmp(s+1, t);
  else
    return strcmp(s+n-m, t);
}

static void faust_any(t_faust *x, t_symbol *s, int argc, t_atom *argv)
{
  if (!x->dsp) return;
  PdUI *ui = x->ui;
  if (s == &s_bang) {
    for (int i = 0; i < ui->nelems; i++)
      if (ui->elems[i].label && ui->elems[i].zone) {
	t_atom args[6];
	t_symbol *_s;
	switch (ui->elems[i].type) {
	case UI_BUTTON:
	  _s = s_button;
	  break;
	case UI_CHECK_BUTTON:
	  _s = s_checkbox;
	  break;
	case UI_V_SLIDER:
	  _s = s_vslider;
	  break;
	case UI_H_SLIDER:
	  _s = s_hslider;
	  break;
	case UI_NUM_ENTRY:
	  _s = s_nentry;
	  break;
	case UI_V_BARGRAPH:
	  _s = s_vbargraph;
	  break;
	case UI_H_BARGRAPH:
	  _s = s_hbargraph;
	  break;
	default:
	  continue;
	}
	SETSYMBOL(&args[0], gensym(ui->elems[i].label));
	SETFLOAT(&args[1], *ui->elems[i].zone);
	SETFLOAT(&args[2], ui->elems[i].init);
	SETFLOAT(&args[3], ui->elems[i].min);
	SETFLOAT(&args[4], ui->elems[i].max);
	SETFLOAT(&args[5], ui->elems[i].step);
	outlet_anything(x->out, _s, 6, args);
      }
  } else {
    const char *label = s->s_name;
    int count = 0;
    for (int i = 0; i < ui->nelems; i++)
      if (ui->elems[i].label &&
	  pathcmp(ui->elems[i].label, label) == 0) {
	if (argc == 0) {
	  if (ui->elems[i].zone) {
	    t_atom arg;
	    SETFLOAT(&arg, *ui->elems[i].zone);
	    outlet_anything(x->out, gensym(ui->elems[i].label), 1, &arg);
	  }
	  ++count;
	} else if (argc == 1 &&
		   (argv[0].a_type == A_FLOAT ||
		    argv[0].a_type == A_DEFFLOAT) &&
		   ui->elems[i].zone) {
	  float f = atom_getfloat(argv);
	  *ui->elems[i].zone = f;
	  ++count;
	} else
	  pd_error(x, "[faust] %s: bad control argument: %s",
		   x->label->c_str(), label);
      }
    if (count == 0 && strcmp(label, "active") == 0) {
      if (argc == 0) {
	t_atom arg;
	SETFLOAT(&arg, (float)x->active);
	outlet_anything(x->out, gensym((char*)"active"), 1, &arg);
      } else if (argc == 1 &&
		 (argv[0].a_type == A_FLOAT ||
		  argv[0].a_type == A_DEFFLOAT)) {
	float f = atom_getfloat(argv);
	x->active = (int)f;
	x->xfade = x->n_xfade;
      }
    }
  }
}

static void faust_free(t_faust *x)
{
  if (x->label) delete x->label;
  if (x->dsp) delete x->dsp;
  if (x->ui) delete x->ui;
  if (x->inputs) free(x->inputs);
  if (x->outputs) free(x->outputs);
  if (x->buf) {
    for (int i = 0; i < x->n_out; i++)
      if (x->buf[i]) free(x->buf[i]);
    free(x->buf);
  }
}

static void *faust_new(t_symbol *s, int argc, t_atom *argv)
{
  t_faust *x = (t_faust*)pd_new(faust_class);
  int sr = -1;
  t_symbol *id = NULL;
  x->active = 1;
  for (int i = 0; i < argc; i++)
    if (argv[i].a_type == A_FLOAT || argv[i].a_type == A_DEFFLOAT)
      sr = (int)argv[i].a_w.w_float;
    else if (argv[i].a_type == A_SYMBOL || argv[i].a_type == A_DEFSYMBOL)
      id = argv[i].a_w.w_symbol;
  x->rate = sr;
  if (sr <= 0) sr = 44100;
  x->xfade = 0; x->n_xfade = (int)(sr*XFADE_TIME/64);
  x->inputs = x->outputs = x->buf = NULL;
  x->label = new string(sym(mydsp) "~");
  x->dsp = new mydsp();
  x->ui = new PdUI(id?id->s_name:NULL);
  if (!x->dsp || !x->ui || !x->label) goto error;
  if (id) {
    *x->label += " ";
    *x->label += id->s_name;
  }
  x->n_in = x->dsp->getNumInputs();
  x->n_out = x->dsp->getNumOutputs();
  if (x->n_in > 0)
    x->inputs = (t_sample**)malloc(x->n_in*sizeof(t_sample*));
  if (x->n_out > 0) {
    x->outputs = (t_sample**)malloc(x->n_out*sizeof(t_sample*));
    x->buf = (t_sample**)malloc(x->n_out*sizeof(t_sample*));
  }
  if ((x->n_in > 0 && x->inputs == NULL) ||
      (x->n_out > 0 && (x->outputs == NULL || x->buf == NULL)))
    goto error;
  for (int i = 0; i < x->n_out; i++)
    x->buf[i] = NULL;
  x->dsp->init(sr);
  x->dsp->buildUserInterface(x->ui);
  for (int i = 0; i < x->n_in; i++)
    inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
  x->out = outlet_new(&x->x_obj, 0);
  for (int i = 0; i < x->n_out; i++)
    outlet_new(&x->x_obj, &s_signal);
  return (void *)x;
 error:
  faust_free(x);
  x->dsp = NULL; x->ui = NULL;
  x->inputs = x->outputs = x->buf = NULL;
  return (void *)x;
}

extern "C" void faust_setup(mydsp)
{
  t_symbol *s = gensym(sym(mydsp) "~");
  faust_class =
    class_new(s, (t_newmethod)faust_new, (t_method)faust_free,
	      sizeof(t_faust), CLASS_DEFAULT,
	      A_GIMME, A_NULL);
  class_addmethod(faust_class, (t_method)faust_dsp, gensym((char*)"dsp"), A_NULL);
  class_addanything(faust_class, faust_any);
  class_addmethod(faust_class, nullfn, &s_signal, A_NULL);
  s_button = gensym((char*)"button");
  s_checkbox = gensym((char*)"checkbox");
  s_vslider = gensym((char*)"vslider");
  s_hslider = gensym((char*)"hslider");
  s_nentry = gensym((char*)"nentry");
  s_vbargraph = gensym((char*)"vbargraph");
  s_hbargraph = gensym((char*)"hbargrap");
  /* give some indication that we're loaded and ready to go */
  mydsp dsp = mydsp();
  post("[faust] %s: %d inputs, %d outputs", sym(mydsp) "~",
       dsp.getNumInputs(), dsp.getNumOutputs());
}
